@@ -12,7 +12,7 @@ require 'puppet/external/event-loop'
 require 'puppet/util'
 require 'puppet/util/log'
 require 'puppet/util/autoload'
-require 'puppet/util/config'
+require 'puppet/util/settings'
 require 'puppet/util/feature'
 require 'puppet/util/suidmanager'
 
@@ -25,7 +25,7 @@ require 'puppet/util/suidmanager'
 # it's also a place to find top-level commands like 'debug'
 
 module Puppet
-    PUPPETVERSION = '0.23.0'
+    PUPPETVERSION = '0.24.9'
 
     def Puppet.version
         return PUPPETVERSION
@@ -45,7 +45,7 @@ module Puppet
     end
 
     # the hash that determines how our system behaves
-    @@config = Puppet::Util::Config.new
+    @@settings = Puppet::Util::Settings.new
 
     # The services running in this process.
     @services ||= []
@@ -76,7 +76,7 @@ module Puppet
 
     # Store a new default value.
     def self.setdefaults(section, hash)
-        @@config.setdefaults(section, hash)
+        @@settings.setdefaults(section, hash)
     end
 
 	# configuration parameter access and stuff
@@ -89,17 +89,17 @@ module Puppet
                 return false
             end
         else
-            return @@config[param]
+            return @@settings[param]
         end
 	end
 
 	# configuration parameter access and stuff
 	def self.[]=(param,value)
-        @@config[param] = value
+        @@settings[param] = value
 	end
 
     def self.clear
-        @@config.clear
+        @@settings.clear
     end
 
     def self.debug=(value)
@@ -110,58 +110,17 @@ module Puppet
         end
     end
 
-    def self.config
-        @@config
+    def self.settings
+        @@settings
     end
 
     # Load all of the configuration parameters.
-    require 'puppet/configuration'
+    require 'puppet/defaults'
 
-    def self.genconfig
-        if Puppet[:configprint] != ""
-            val = Puppet[:configprint]
-            if val == "all"
-                hash = {}
-                Puppet.config.each do |name, obj|
-                    val = obj.value
-                    case val
-                    when true, false, "": val = val.inspect
-                    end
-                    hash[name] = val
-                end
-                hash.sort { |a,b| a[0].to_s <=> b[0].to_s }.each do |name, val|
-                    puts "%s = %s" % [name, val]
-                end
-            elsif val =~ /,/
-                val.split(/\s*,\s*/).sort.each do |v|
-                    if Puppet.config.include?(v)
-                        puts "%s = %s" % [v, Puppet[v]]
-                    else
-                        puts "invalid parameter: %s" % v
-                        exit(1)
-                    end
-                end
-            else
-                val.split(/\s*,\s*/).sort.each do |v|
-                    if Puppet.config.include?(v)
-                        puts Puppet[val]
-                    else
-                        puts "invalid parameter: %s" % v
-                        exit(1)
-                    end
-                end
-            end
-            exit(0)
-        end
-        if Puppet[:genconfig]
-            puts Puppet.config.to_config
-            exit(0)
-        end
-    end
 
     def self.genmanifest
         if Puppet[:genmanifest]
-            puts Puppet.config.to_manifest
+            puts Puppet.settings.to_manifest
             exit(0)
         end
     end
@@ -208,14 +167,14 @@ module Puppet
         oldconfig ||= File.join(Puppet[:confdir], Puppet[:name].to_s + ".conf")
         if FileTest.exists?(oldconfig) and Puppet[:name] != "puppet"
             Puppet.warning "Individual config files are deprecated; remove %s and use puppet.conf" % oldconfig
-            Puppet.config.old_parse(oldconfig)
+            Puppet.settings.old_parse(oldconfig)
             return
         end
 
         # Now check for the normal config.
         if Puppet[:config] and File.exists? Puppet[:config]
             Puppet.debug "Parsing %s" % Puppet[:config]
-            Puppet.config.parse(Puppet[:config])
+            Puppet.settings.parse(Puppet[:config])
         end
     end
 
@@ -242,7 +201,7 @@ module Puppet
 
         # Handle restarting.
         trap(:HUP) do
-            if client = @services.find { |s| s.is_a? Puppet::Network::Client::Master } and client.running?
+            if client = @services.find { |s| s.is_a? Puppet::Network::Client.master } and client.running?
                 client.restart
             else
                 Puppet.restart
@@ -421,11 +380,11 @@ module Puppet
 end
 
 require 'puppet/type'
-require 'puppet/modules'
+require 'puppet/network'
+require 'puppet/module'
 require 'puppet/util/storage'
 require 'puppet/parser/interpreter'
 if Puppet[:storeconfigs]
     require 'puppet/rails'
 end
 
-# $Id: puppet.rb 2631 2007-06-19 23:12:23Z luke $
