@@ -69,8 +69,6 @@ module Puppet
         :rundir => { 
             :default => rundir,
             :mode => 01777,
-            :owner => "$user",
-            :group => "$group",
             :desc => "Where Puppet PID files are kept."
         },
         :genconfig => [false,
@@ -97,8 +95,17 @@ module Puppet
         :path => {:default => "none",
             :desc => "The shell search path.  Defaults to whatever is inherited
                 from the parent process.",
+            :call_on_define => true, # Call our hook with the default value, so we always get the libdir set.
             :hook => proc do |value|
                 ENV["PATH"] = value unless value == "none"
+
+                paths = ENV["PATH"].split(File::PATH_SEPARATOR)
+                %w{/usr/sbin /sbin}.each do |path|
+                    unless paths.include?(path)
+                        ENV["PATH"] += File::PATH_SEPARATOR + path
+                    end
+                end
+                value
             end
         },
         :libdir => {:default => "$vardir/lib",
@@ -124,16 +131,11 @@ module Puppet
             namespaces and methods.  This can be used as a coarse-grained
             authorization system for both ``puppetd`` and ``puppetmasterd``."
         ],
-        :environments => ["production,development", "The valid environments for Puppet clients.
-            This is more useful as a server-side setting than client, but any
-            environment chosen must be in this list.  Values should be
-            separated by a comma."],
-        :environment => {:default => "development", :desc => "The environment Puppet is running in.  For clients
+        :environment => {:default => "production", :desc => "The environment Puppet is running in.  For clients
             (e.g., ``puppetd``) this determines the environment itself, which
             is used to find modules and much more.  For servers (i.e.,
             ``puppetmasterd``) this provides the default environment for nodes
-            we know nothing about.",
-            :hook => proc { |value| raise(ArgumentError, "Invalid environment %s" % value) unless Puppet::Node::Environment.valid?(value) }
+            we know nothing about."
         },
         :diff_args => ["", "Which arguments to pass to the diff command when printing differences between files."],
         :diff => ["diff", "Which diff command to use when printing differences between files."],
@@ -160,8 +162,12 @@ module Puppet
     end
 
     Puppet.setdefaults(:ssl,
-        :certname => [fqdn, "The name to use when handling certificates.  Defaults
-            to the fully qualified domain name."],
+        # We have to downcase the fqdn, because the current ssl stuff (as oppsed to in master) doesn't have good facilities for 
+        # manipulating naming.
+        :certname => {:default => fqdn.downcase, :desc => "The name to use when handling certificates.  Defaults
+            to the fully qualified domain name.",
+            :call_on_define => true, # Call our hook with the default value, so we're always downcased
+            :hook => proc { |value| raise(ArgumentError, "Certificate names must be lower case; see #1168") unless value == value.downcase }},
         :certdnsnames => ['', "The DNS names on the Server certificate as a colon-separated list.
             If it's anything other than an empty string, it will be used as an alias in the created
             certificate.  By default, only the server gets an alias set up, and only for 'puppet'."],
@@ -354,7 +360,9 @@ module Puppet
         # To make sure this directory is created before we try to use it on the server, we need
         # it to be in the server section (#1138).
         :yamldir => {:default => "$vardir/yaml", :owner => "$user", :group => "$user", :mode => "750",
-            :desc => "The directory in which YAML data is stored, usually in a subdirectory."}
+            :desc => "The directory in which YAML data is stored, usually in a subdirectory."},
+        :clientyamldir => {:default => "$vardir/client_yaml", :mode => "750",
+            :desc => "The directory in which client-side YAML data is stored."}
     )
 
     self.setdefaults(:puppetd,
@@ -412,7 +420,10 @@ module Puppet
         :ca_server => ["$server", "The server to use for certificate
             authority requests.  It's a separate server because it cannot
             and does not need to horizontally scale."],
-        :ca_port => ["$masterport", "The port to use for the certificate authority."]
+        :ca_port => ["$masterport", "The port to use for the certificate authority."],
+        :catalog_format => ["yaml", "What format to use to dump the catalog.  Only supports
+            'marshal' and 'yaml'.  Only matters on the client, since it asks the server
+            for a specific format."]
     )
         
     self.setdefaults(:filebucket,
@@ -505,9 +516,11 @@ module Puppet
 
     # Central fact information.
     self.setdefaults(:main,
-        :factpath => ["$vardir/facts",
-            "Where Puppet should look for facts.  Multiple directories should
-            be colon-separated, like normal PATH variables."],
+        :factpath => {:default => "$vardir/facts",
+            :desc => "Where Puppet should look for facts.  Multiple directories should
+                be colon-separated, like normal PATH variables.",
+            :call_on_define => true, # Call our hook with the default value, so we always get the value added to facter.
+            :hook => proc { |value| Facter.search(value) if Facter.respond_to?(:search) }},
         :factdest => ["$vardir/facts",
             "Where Puppet should store facts that it pulls down from the central
             server."],
@@ -622,6 +635,10 @@ module Puppet
         :ldapclassattrs => ["puppetclass",
             "The LDAP attributes to use to define Puppet classes.  Values
             should be comma-separated."],
+        :ldapstackedattrs => ["puppetvar",
+            "The LDAP attributes that should be stacked to arrays by adding
+            the values in all hierarchy elements of the tree.  Values
+            should be comma-separated."],
         :ldapattrs => ["all",
             "The LDAP attributes to include when querying LDAP for nodes.  All
             returned attributes are set as variables in the top-level scope.
@@ -651,7 +668,8 @@ module Puppet
     setdefaults(:parser,
         :lexical => [false, "Whether to use lexical scoping (vs. dynamic)."],
         :templatedir => ["$vardir/templates",
-            "Where Puppet looks for template files."
+            "Where Puppet looks for template files.  Can be a list of colon-seperated
+             directories."
         ]
     )
 
