@@ -23,8 +23,8 @@ class Puppet::Network::Handler
 
         # Tell a client whether there's a fresh config for it
         def freshness(client = nil, clientip = nil)
-            client ||= Facter.value("hostname")
-            config_handler.version(client, clientip)
+            # Always force a recompile.  Newer clients shouldn't do this (as of April 2008).
+            return Time.now.to_i
         end
 
         def initialize(hash = {})
@@ -51,53 +51,27 @@ class Puppet::Network::Handler
             if hash.include?(:Classes)
                 args[:Classes] = hash[:Classes]
             end
-
-            @config_handler = Puppet::Network::Handler.handler(:configuration).new(args)
         end
 
         # Call our various handlers; this handler is getting deprecated.
         def getconfig(facts, format = "marshal", client = nil, clientip = nil)
             facts = decode_facts(facts)
-            client, clientip = clientname(client, clientip, facts)
+
+            client ||= facts["hostname"]
 
             # Pass the facts to the fact handler
             Puppet::Node::Facts.new(client, facts).save unless local?
 
-            # And get the configuration from the config handler
-            config = nil
-            benchmark(:notice, "Compiled configuration for %s" % client) do
-                config = config_handler.configuration(client)
+            catalog = Puppet::Node::Catalog.find(client)
+
+            case format
+            when "yaml":
+                return CGI.escape(catalog.extract.to_yaml(:UseBlock => true))
+            when "marshal":
+                return CGI.escape(Marshal.dump(catalog.extract))
+            else
+                raise "Invalid markup format '%s'" % format
             end
-
-            return translate(config.extract)
-        end
-
-        private
-
-        # Manipulate the client name as appropriate.
-        def clientname(name, ip, facts)
-            # Always use the hostname from Facter.
-            client = facts["hostname"]
-            clientip = facts["ipaddress"]
-            if Puppet[:node_name] == 'cert'
-                if name
-                    client = name
-                    facts["fqdn"] = client
-                    facts["hostname"], facts["domain"] = client.split('.', 2)
-                end
-                if ip
-                    clientip = ip
-                end
-            end
-
-            return client, clientip
-        end
-
-        def config_handler
-            unless defined? @config_handler
-                @config_handler = Puppet::Network::Handler.handler(:config).new :local => local?
-            end
-            @config_handler
         end
 
         # 
@@ -123,11 +97,6 @@ class Puppet::Network::Handler
 
         # Translate our configuration appropriately for sending back to a client.
         def translate(config)
-            if local?
-                config
-            else
-                CGI.escape(config.to_yaml(:UseBlock => true))
-            end
         end
     end
 end
