@@ -1,10 +1,19 @@
 require 'puppet/util/methodhelper'
+require 'puppet/util/log_paths'
+require 'puppet/util/logging'
+require 'puppet/util/docs'
 
-class Puppet::Parameter < Puppet::Element
+class Puppet::Parameter
+    include Puppet::Util
+    include Puppet::Util::Errors
+    include Puppet::Util::LogPaths
+    include Puppet::Util::Logging
     include Puppet::Util::MethodHelper
     class << self
+        include Puppet::Util
+        include Puppet::Util::Docs
         attr_reader :validater, :munger, :name, :default, :required_features
-        attr_accessor :metaparam, :element
+        attr_accessor :metaparam
 
         # Define the default value for a given parameter or parameter.  This
         # means that 'nil' is an invalid default value.  This defines
@@ -54,7 +63,7 @@ class Puppet::Parameter < Puppet::Element
                     end
                     unless regs.empty?
                         @doc += "  Values can also match ``" +
-                            regs.join("``, ``") + "``."
+                            regs.collect { |r| r.inspect }.join("``, ``") + "``."
                     end
                 end
 
@@ -280,9 +289,6 @@ class Puppet::Parameter < Puppet::Element
             raise Puppet::DevError, "No resource set for %s" % self.class.name
         end
 
-        # LAK 2007-05-09: Keep the @parent around for backward compatibility.
-        #@parent = @resource
-
         if ! self.metaparam? and klass = Puppet::Type.metaparamclass(self.class.name)
             setup_shadow(klass)
         end
@@ -369,36 +375,37 @@ class Puppet::Parameter < Puppet::Element
         vals = self.class.values
         regs = self.class.regexes
 
-        if regs.is_a? Hash # this is true on properties
-            regs = regs.keys
-        end
-        if vals.empty? and regs.empty?
-            # This parameter isn't using defined values to do its work.
-            return 
-        end
+        # this is true on properties
+        regs = regs.keys if regs.is_a?(Hash)
+
+        # This parameter isn't using defined values to do its work.
+        return if vals.empty? and regs.empty?
+
         newval = value
-        unless value.is_a?(Symbol)
-            newval = value.to_s.intern
+        newval = value.to_s.intern unless value.is_a?(Symbol)
+
+        name = newval
+
+        unless vals.include?(newval) or name = self.class.alias(newval) or name = self.class.match?(value) # We match the string, not the symbol
+            str = "Invalid '%s' value %s. " %
+                [self.class.name, value.inspect]
+
+            unless vals.empty?
+                str += "Valid values are %s. " % vals.join(", ")
+            end
+
+            unless regs.empty?
+                str += "Valid values match %s." % regs.collect { |r|
+                    r.to_s
+                }.join(", ")
+            end
+
+            raise ArgumentError, str
         end
 
-        unless vals.include?(newval) or
-            self.class.alias(newval) or
-            self.class.match?(value) # We match the string, not the symbol
-                str = "Invalid '%s' value %s. " %
-                    [self.class.name, value.inspect]
-
-                unless vals.empty?
-                    str += "Valid values are %s. " % vals.join(", ")
-                end
-
-                unless regs.empty?
-                    str += "Valid values match %s." % regs.collect { |r|
-                        r.to_s
-                    }.join(", ")
-                end
-
-                raise ArgumentError, str
-        end
+        # Now check for features.
+        name = name[0] if name.is_a?(Array) # This is true for regexes.
+        validate_features_per_value(name) if is_a?(Puppet::Property)
     end
 
     def remove
@@ -406,39 +413,12 @@ class Puppet::Parameter < Puppet::Element
         @shadow = nil
     end
 
-    # This should only be called for parameters, but go ahead and make
-    # it possible to call for properties, too.
-    def value
-        if self.is_a?(Puppet::Property)
-            # We should return the 'is' value if there's not 'should'
-            # value.  This might be bad, though, because the 'should'
-            # method knows whether to return an array or not and that info
-            # is not exposed, and the 'is' value could be a symbol.  I
-            # can't seem to create a test in which this is a problem, but
-            # that doesn't mean it's not one.
-            if self.should
-                return self.should
-            else
-                return self.retrieve
-            end
-        else
-            if defined? @value
-                return @value
-            else
-                return nil
-            end
-        end
-    end
+    attr_reader :value
 
     # Store the value provided.  All of the checking should possibly be
     # late-binding (e.g., users might not exist when the value is assigned
     # but might when it is asked for).
     def value=(value)
-        # If we're a parameter, just hand the processing off to the should
-        # method.
-        if self.is_a?(Puppet::Property)
-            return self.should = value
-        end
         if respond_to?(:validate)
             validate(value)
         end
@@ -475,6 +455,12 @@ class Puppet::Parameter < Puppet::Element
     def to_s
         s = "Parameter(%s)" % self.name
     end
+
+    # Make sure that we've got all of the required features for a given value.
+    def validate_features_per_value(value)
+        if features = self.class.value_option(value, :required_features)
+            raise ArgumentError, "Provider must have features '%s' to set '%s' to '%s'" % [features, self.class.name, value] unless provider.satisfies?(features)
+        end
+    end
 end
 
-# $Id: parameter.rb 2503 2007-05-10 21:46:59Z luke $
