@@ -1,8 +1,11 @@
 require 'puppet/provider/nameservice/objectadd'
 
 Puppet::Type.type(:group).provide :groupadd, :parent => Puppet::Provider::NameService::ObjectAdd do
-    desc "Group management via ``groupadd`` and its ilk.  The default
-        for most platforms"
+    desc "Group management via ``groupadd`` and its ilk.
+
+    The default for most platforms
+
+  "
 
     commands :add => "groupadd", :delete => "groupdel", :modify => "groupmod"
 
@@ -17,7 +20,7 @@ Puppet::Type.type(:group).provide :groupadd, :parent => Puppet::Provider::NameSe
                 cmd << flag(:gid) << gid
             end
         end
-        if @resource[:allowdupe] == :true
+        if @resource.allowdupe?
             cmd << "-o"
         end
         cmd << @resource[:name]
@@ -26,4 +29,3 @@ Puppet::Type.type(:group).provide :groupadd, :parent => Puppet::Provider::NameSe
     end
 end
 
-# $Id: groupadd.rb 2501 2007-05-09 23:08:42Z luke $
