@@ -1,11 +1,15 @@
-# Manage NetInfo POSIX objects.  Probably only used on OS X, but I suppose
-# it could be used elsewhere.
+# Manage NetInfo POSIX objects.
+#
+# This provider has been deprecated. You should be using the directoryservice
+# nameservice provider instead.
+
 require 'puppet/provider/nameservice/netinfo'
 
 Puppet::Type.type(:group).provide :netinfo, :parent => Puppet::Provider::NameService::NetInfo do
-    desc "Group management using NetInfo."
+    desc "Group management using NetInfo.
+
+  "
     commands :nireport => "nireport", :niutil => "niutil"
 
-    defaultfor :operatingsystem => :darwin
 end
 
