@@ -1,17 +1,19 @@
 require 'puppet'
 require 'puppet/rails/param_name'
+require 'puppet/rails/param_value'
 require 'puppet/rails/puppet_tag'
 require 'puppet/util/rails/collection_merger'
 
 class Puppet::Rails::Resource < ActiveRecord::Base
     include Puppet::Util::CollectionMerger
+    include Puppet::Util::ReferenceSerializer
 
-    has_many :param_values, :dependent => :destroy
-    has_many :param_names, :through => :param_values
+    has_many :param_values, :dependent => :destroy, :class_name => "Puppet::Rails::ParamValue"
+    has_many :param_names, :through => :param_values, :class_name => "Puppet::Rails::ParamName"
+
+    has_many :resource_tags, :dependent => :destroy, :class_name => "Puppet::Rails::ResourceTag"
+    has_many :puppet_tags, :through => :resource_tags, :class_name => "Puppet::Rails::PuppetTag"
 
-    has_many :resource_tags, :dependent => :destroy
-    has_many :puppet_tags, :through => :resource_tags
-    
     belongs_to :source_file
     belongs_to :host
 
@@ -32,21 +34,46 @@ class Puppet::Rails::Resource < ActiveRecord::Base
         self.source_file = Puppet::Rails::SourceFile.find_or_create_by_filename(file)
     end
 
+    def title
+        unserialize_value(self[:title])
+    end
+
+    def add_param_to_hash(param)
+        @params_hash ||= []
+        @params_hash << param
+    end
+
+    def add_tag_to_hash(tag)
+        @tags_hash ||= []
+        @tags_hash << tag
+    end
+
+    def params_hash=(hash)
+        @params_hash = hash
+    end
+
+    def tags_hash=(hash)
+        @tags_hash = hash
+    end
+
     # returns a hash of param_names.name => [param_values]
     def get_params_hash(values = nil)
-        values ||= param_values.find(:all, :include => :param_name)
-        values.inject({}) do | hash, value |
-            hash[value.param_name.name] ||= []
-            hash[value.param_name.name] << value
+        values ||= @params_hash || Puppet::Rails::ParamValue.find_all_params_from_resource(self)
+        if values.size == 0
+            return {}
+        end
+        values.inject({}) do |hash, value|
+            hash[value['name']] ||= []
+            hash[value['name']] << value
             hash
         end
     end
-    
+
     def get_tag_hash(tags = nil)
-        tags ||= resource_tags.find(:all, :include => :puppet_tag)
+        tags ||= @tags_hash || Puppet::Rails::ResourceTag.find_all_tags_from_resource(self)
         return tags.inject({}) do |hash, tag|
             # We have to store the tag object, not just the tag name.
-            hash[tag.puppet_tag.name] = tag
+            hash[tag['name']] = tag
             hash
         end
     end
@@ -73,7 +100,7 @@ class Puppet::Rails::Resource < ActiveRecord::Base
         result = get_params_hash
         result.each do |param, value|
             if value.is_a?(Array)
-                result[param] = value.collect { |v| v.value }
+                result[param] = value.collect { |v| v['value'] }
             else
                 result[param] = value.value
             end
@@ -82,7 +109,7 @@ class Puppet::Rails::Resource < ActiveRecord::Base
     end
 
     def ref
-        "%s[%s]" % [self[:restype].capitalize, self[:title]]
+        "%s[%s]" % [self[:restype].split("::").collect { |s| s.capitalize }.join("::"), self.title.to_s]
     end
 
     # Convert our object to a resource.  Do not retain whether the object
@@ -98,6 +125,7 @@ class Puppet::Rails::Resource < ActiveRecord::Base
         hash.delete("host_id")
         hash.delete("updated_at")
         hash.delete("source_file_id")
+        hash.delete("created_at")
         hash.delete("id")
         hash.each do |p, v|
             hash.delete(p) if v.nil?
