@@ -3,7 +3,7 @@ require 'pp'
 
 require 'net/smtp'
 
-Puppet::Network::Handler.report.newreport(:tagmail) do
+Puppet::Reports.register_report(:tagmail) do
     desc "This report sends specific log messages to specific email addresses
         based on the tags in the log messages.  See the
         `UsingTags tag documentation`:trac: for more information
@@ -28,11 +28,13 @@ Puppet::Network::Handler.report.newreport(:tagmail) do
 
         This will send all messages to ``me@domain.com``, and all messages from
         webservers that are not also from mailservers to ``httpadmins@domain.com``.
+
+        If you are using anti-spam controls, such as grey-listing, on your mail 
+        server you should whitelist the sending email (controlled by ``reportform`` 
+        configuration option) to ensure your email is not discarded as spam.  
         "
 
 
-    Puppet.config.use(:tagmail)
-
     # Find all matching messages.
     def match(taglists)
         reports = []
@@ -127,7 +129,14 @@ Puppet::Network::Handler.report.newreport(:tagmail) do
                     Net::SMTP.start(Puppet[:smtpserver]) do |smtp|
                         reports.each do |emails, messages|
                             Puppet.info "Sending report to %s" % emails.join(", ")
-                            smtp.send_message(messages, Puppet[:reportfrom], *emails)
+                            smtp.open_message_stream(Puppet[:reportfrom], *emails) do |p|
+                              p.puts "From: #{Puppet[:reportfrom]}"
+                              p.puts "Subject: Puppet Report for %s" % self.host
+                              p.puts "To: " + emails.join(", ")
+                              p.puts "Date: " + Time.now.rfc2822
+                              p.puts
+                              p.puts messages
+                            end 
                         end
                     end
                 rescue => detail
@@ -142,12 +151,14 @@ Puppet::Network::Handler.report.newreport(:tagmail) do
                     reports.each do |emails, messages|
                         Puppet.info "Sending report to %s" % emails.join(", ")
                         # We need to open a separate process for every set of email addresses
-                        IO.popen(Puppet[:sendmail] + " " + emails.join(" "), "w") do |p|
-                            p.puts "From: #{Puppet[:reportfrom]}"
-                            p.puts "Subject: Puppet Report for %s" % self.host
-                            p.puts "To: " + emails.join(", ")
+                        sync.synchronize do
+                            IO.popen(Puppet[:sendmail] + " " + emails.join(" "), "w") do |p|
+                                p.puts "From: #{Puppet[:reportfrom]}"
+                                p.puts "Subject: Puppet Report for %s" % self.host
+                                p.puts "To: " + emails.join(", ")
 
-                            p.puts messages
+                                p.puts messages
+                            end
                         end
                     end
                 rescue => detail
@@ -165,6 +176,12 @@ Puppet::Network::Handler.report.newreport(:tagmail) do
         # Don't bother waiting for the pid to return.
         Process.detach(pid)
     end
+
+    def sync
+        unless defined?(@sync)
+            @sync = Sync.new
+        end
+        @sync
+    end
 end
 
-# $Id: tagmail.rb 2463 2007-05-04 23:09:34Z luke $
