@@ -6,7 +6,12 @@ module Puppet
         @doc = "Executes external commands.  It is critical that all commands
             executed using this mechanism can be run multiple times without
             harm, i.e., they are *idempotent*.  One useful way to create idempotent
-            commands is to use the *creates* parameter.
+            commands is to use the checks like ``creates`` to avoid running the
+            command unless some condition is met.
+
+            Note also that you can restrict an ``exec`` to only run when it receives
+            events by using the ``refreshonly`` parameter; this is a useful way to
+            have your configuration respond to events with arbitrary commands.
 
             It is worth noting that ``exec`` is special, in that it is not
             currently considered an error to have multiple ``exec`` instances
@@ -48,10 +53,9 @@ module Puppet
             to native Puppet types as quickly as possible.  If you find that
             you are doing a lot of work with ``exec``, please at least notify
             us at Reductive Labs what you are doing, and hopefully we can work with
-            you to get a native element type for the work you are doing."
+            you to get a native resource type for the work you are doing."
 
         require 'open3'
-        require 'puppet/type/property'
 
         # Create a new check mechanism.  It's basically just a parameter that
         # provides one extra 'check' method.
@@ -109,15 +113,16 @@ module Puppet
                     self.fail "Command exceeded timeout" % value.inspect
                 end
 
-                loglevel = @resource[:loglevel]
-                if status.exitstatus.to_s != self.should.to_s
-                    self.fail("%s returned %s instead of %s" %
-                        [self.resource[:command], status.exitstatus, self.should.to_s])
-                end
-
                 if log = @resource[:logoutput]
-                    if log == :true
+                    case log 
+                    when :true 
                         log = @resource[:loglevel]
+                    when :on_failure
+                        if status.exitstatus.to_s != self.should.to_s
+                            log = @resource[:loglevel]
+                        else
+                            log = :false
+                        end
                     end
                     unless log == :false
                         @output.split(/\n/).each { |line|
@@ -126,6 +131,11 @@ module Puppet
                     end
                 end
 
+                if status.exitstatus.to_s != self.should.to_s
+                    self.fail("%s returned %s instead of %s" %
+                        [self.resource[:command], status.exitstatus, self.should.to_s])
+                end
+
                 return event
             end
         end
@@ -197,13 +207,11 @@ module Puppet
 
         newparam(:logoutput) do
             desc "Whether to log output.  Defaults to logging output at the
-                loglevel for the ``exec`` element.  Values are **true**, *false*,
-                and any legal log level."
+                loglevel for the ``exec`` resource. Use *on_failure* to only
+                log the output when the command reports an error.  Values are
+                **true**, *false*, *on_failure*, and any legal log level."
 
-            values = [:true, :false]
-            # And all of the log levels
-            Puppet::Util::Log.eachlevel { |level| values << level }
-            newvalues(*values)
+            newvalues(:true, :false, :on_failure)
         end
 
         newparam(:refresh) do
@@ -218,6 +226,15 @@ module Puppet
         end
 
         newparam(:env) do
+            desc "This parameter is deprecated. Use 'environment' instead."
+
+            munge do |value|
+                warning "'env' is deprecated on exec; use 'environment' instead."
+                resource[:environment] = value
+            end
+        end
+
+        newparam(:environment) do
             desc "Any additional environment variables you want to set for a
                 command.  Note that if you use this to set PATH, it will override
                 the ``path`` attribute.  Multiple environment variables should be
@@ -268,12 +285,12 @@ module Puppet
                     # Rebuild the database, but only when the file changes
                     exec { newaliases:
                         path => [\"/usr/bin\", \"/usr/sbin\"],
-                        subscribe => file[\"/etc/aliases\"],
+                        subscribe => File[\"/etc/aliases\"],
                         refreshonly => true
                     }
                 
-                Note that only ``subscribe`` can trigger actions, not ``require``,
-                so it only makes sense to use ``refreshonly`` with ``subscribe``."
+                Note that only ``subscribe`` and ``notify`` can trigger actions, not ``require``,
+                so it only makes sense to use ``refreshonly`` with ``subscribe`` or ``notify``."
 
             newvalues(:true, :false)
 
@@ -373,6 +390,12 @@ module Puppet
 
                 Note that this command follows the same rules as the main command,
                 which is to say that it must be fully qualified if the path is not set.
+
+                Also note that onlyif can take an array as its value, eg::
+
+                    onlyif => [\"test -f /tmp/file1\", \"test -f /tmp/file2\"]
+
+                This will only run the exec if /all/ conditions in the array return true.
                 "
 
             validate do |cmds|
@@ -543,32 +566,32 @@ module Puppet
             begin
                 # Do our chdir
                 Dir.chdir(dir) do
-                    env = {}
+                    environment = {}
 
                     if self[:path]
-                        env[:PATH] = self[:path].join(":")
+                        environment[:PATH] = self[:path].join(":")
                     end
 
-                    if envlist = self[:env]
+                    if envlist = self[:environment]
                         envlist = [envlist] unless envlist.is_a? Array
                         envlist.each do |setting|
                             if setting =~ /^(\w+)=((.|\n)+)$/
                                 name = $1
                                 value = $2
-                                if env.include? name
+                                if environment.include? name
                                     warning(
                                     "Overriding environment setting '%s' with '%s'" %
                                         [name, value]
                                     )
                                 end
-                                env[name] = value
+                                environment[name] = value
                             else
-                                warning "Cannot understand env setting %s" % setting.inspect
+                                warning "Cannot understand environment setting %s" % setting.inspect
                             end
                         end
                     end
 
-                    withenv env do
+                    withenv environment do
                         Timeout::timeout(self[:timeout]) do
                             output, status = Puppet::Util::SUIDManager.run_and_capture(
                                 [command], self[:user], self[:group]
@@ -598,4 +621,3 @@ module Puppet
     end
 end
 
-# $Id: exec.rb 2551 2007-06-04 20:37:14Z luke $
