@@ -72,11 +72,11 @@ class TestTransactions < Test::Unit::TestCase
         path1 = tempfile()
         path2 = tempfile()
         objects = []
-        objects << Puppet::Type.newfile(
+        objects << Puppet::Type.type(:file).create(
             :path => path1,
             :content => "yayness"
         )
-        objects << Puppet::Type.newfile(
+        objects << Puppet::Type.type(:file).create(
             :path => path2,
             :content => "booness"
         )
@@ -155,7 +155,7 @@ class TestTransactions < Test::Unit::TestCase
         path = tempfile()
         firstpath = tempfile()
         secondpath = tempfile()
-        file = Puppet::Type.newfile(:title => "file", :path => path, :content => "yayness")
+        file = Puppet::Type.type(:file).create(:title => "file", :path => path, :content => "yayness")
         first = Puppet::Type.newexec(:title => "first",
                                      :command => "/bin/echo first > #{firstpath}",
                                      :subscribe => [:file, path],
@@ -248,7 +248,7 @@ class TestTransactions < Test::Unit::TestCase
 
         assert_nothing_raised() {
             check.each { |property|
-                value = file.value(property)
+                value = file.property(property).retrieve
                 assert(value)
                 properties[property] = value
             }
@@ -716,6 +716,45 @@ class TestTransactions < Test::Unit::TestCase
             "Not all resources were evaluated or not in the right order")
     end
 
+    # We need to generate resources before we prefetch them, else generated
+    # resources that require prefetching don't work.
+    def test_generate_before_prefetch
+        config = mk_catalog()
+        trans = Puppet::Transaction.new(config)
+
+        generate = nil
+        prefetch = nil
+        trans.expects(:generate).with { |*args| generate = Time.now; true }
+        trans.expects(:prefetch).with { |*args| ! generate.nil? }
+        trans.prepare
+        return
+
+        resource = Puppet::Type.type(:file).create :ensure => :present, :path => tempfile()
+        other_resource = mock 'generated'
+        def resource.generate
+            [other_resource]
+        end
+
+
+        config = mk_catalog(yay, rah)
+        trans = Puppet::Transaction.new(config)
+        
+        assert_nothing_raised do
+            trans.generate
+        end
+        
+        %w{ya ra y r}.each do |name|
+            assert(trans.catalog.vertex?(Puppet::Type.type(:generator)[name]),
+                "Generated %s was not a vertex" % name)
+            assert($finished.include?(name), "%s was not finished" % name)
+        end
+        
+        # Now make sure that cleanup gets rid of those generated types.
+        assert_nothing_raised do
+            trans.cleanup
+        end
+    end
+
     def test_ignore_tags?
         config = Puppet::Node::Catalog.new
         config.host_config = true
@@ -757,7 +796,7 @@ class TestTransactions < Test::Unit::TestCase
     def test_proxy_resources
         type = mkreducer do
             def evaluate
-                return Puppet::PropertyChange.new(Fakeprop.new(
+                return Puppet::Transaction::Change.new(Fakeprop.new(
                     :path => :path, :is => :is, :should => :should, :name => self.name, :resource => "a parent"), :is)
             end
         end
@@ -776,7 +815,7 @@ class TestTransactions < Test::Unit::TestCase
         assert(changes.length > 0, "did not get any changes")
         
         changes.each do |change|
-            assert_equal(resource, change.source, "change did not get proxy set correctly")
+            assert_equal(resource, change.resource, "change did not get proxy set correctly")
         end
     end
     
@@ -866,17 +905,16 @@ class TestTransactions < Test::Unit::TestCase
         assert(result, "c did not trigger anything")
         assert_instance_of(Array, result)
         event = result.shift
-        assert_instance_of(Puppet::Event, event)
-        assert_equal(:triggered, event.event, "event was not set correctly")
+        assert_instance_of(Puppet::Transaction::Event, event)
+        assert_equal(:triggered, event.name, "event was not set correctly")
         assert_equal(c, event.source, "source was not set correctly")
-        assert_equal(trans, event.transaction, "transaction was not set correctly")
 
         assert(trans.triggered?(c, :refresh),
             "Transaction did not store the trigger")
     end
     
     def test_set_target
-        file = Puppet::Type.newfile(:path => tempfile(), :content => "yay")
+        file = Puppet::Type.type(:file).create(:path => tempfile(), :content => "yay")
         exec1 = Puppet::Type.type(:exec).create :command => "/bin/echo exec1"
         exec2 = Puppet::Type.type(:exec).create :command => "/bin/echo exec2"
         trans = Puppet::Transaction.new(mk_catalog(file, exec1, exec2))
@@ -911,7 +949,7 @@ class TestTransactions < Test::Unit::TestCase
             Puppet::Type.rmtype(:norefresh)
         end
 
-        file = Puppet::Type.newfile :path => tempfile(), :content => "yay"
+        file = Puppet::Type.type(:file).create :path => tempfile(), :content => "yay"
         one = klass.create :name => "one", :subscribe => file
         
         assert_apply(file, one)
@@ -956,7 +994,7 @@ class TestTransactions < Test::Unit::TestCase
             newparam(:name) {}
             newproperty(:testing) do
                 def sync
-                    self.is = self.should
+                    # noop
                     :ran_testing
                 end
             end
@@ -989,8 +1027,8 @@ class TestTransactions < Test::Unit::TestCase
         # Now files
         d = tempfile()
         f = File.join(d, "file")
-        file = Puppet::Type.newfile(:path => f, :content => "yay")
-        dir = Puppet::Type.newfile(:path => d, :ensure => :directory, :require => file)
+        file = Puppet::Type.type(:file).create(:path => f, :content => "yay")
+        dir = Puppet::Type.type(:file).create(:path => d, :ensure => :directory, :require => file)
         
         rels[dir] = file
         rels.each do |after, before|
@@ -1014,7 +1052,7 @@ class TestTransactions < Test::Unit::TestCase
         path = tempfile
         epath = tempfile
         spath = tempfile
-        file = Puppet::Type.newfile(:path => path, :ensure => :file,
+        file = Puppet::Type.type(:file).create(:path => path, :ensure => :file,
             :title => "file")
         exec = Puppet::Type.type(:exec).create(:command => "touch %s" % epath,
             :path => ENV["PATH"], :subscribe => file, :refreshonly => true,
@@ -1052,7 +1090,7 @@ class TestTransactions < Test::Unit::TestCase
         3.times do |i|
             path = tempfile
             paths << path
-            file = Puppet::Type.newfile(:path => path, :ensure => :absent,
+            file = Puppet::Type.type(:file).create(:path => path, :ensure => :absent,
                 :backup => false, :title => "file%s" % i)
             File.open(path, "w") { |f| f.puts "" }
             files << file
