@@ -1,6 +1,6 @@
 #!/usr/bin/env ruby
 
-$:.unshift("../../lib") if __FILE__ =~ /\.rb$/
+require File.dirname(__FILE__) + '/../../lib/puppettest'
 
 require 'puppettest'
 require 'mocha'
@@ -131,27 +131,60 @@ class TestType < Test::Unit::TestCase
         }
     end
 
-    # Verify that aliasing works
-    def test_aliasing
-        file = tempfile()
+    def test_aliases_to_self_are_not_failures
+        resource = Puppet.type(:file).create(
+            :name => "/path/to/some/missing/file",
+            :ensure => "file"
+        )
+        resource.stubs(:path).returns("")
 
-        baseobj = nil
-        assert_nothing_raised {
-            baseobj = Puppet.type(:file).create(
-                :name => file,
-                :ensure => "file",
-                :alias => ["funtest"]
-            )
-        }
+        catalog = stub 'catalog'
+        catalog.expects(:resource).with(:file, "/path/to/some/missing/file").returns(resource)
+        resource.catalog = catalog
 
         # Verify our adding ourselves as an alias isn't an error.
-        assert_nothing_raised {
-            baseobj[:alias] = file
+        assert_nothing_raised("Could not add alias") {
+            resource[:alias] = "/path/to/some/missing/file"
         }
 
-        assert_instance_of(Puppet.type(:file), Puppet.type(:file)["funtest"],
-            "Could not retrieve alias")
+        assert_equal(resource.object_id, Puppet.type(:file)["/path/to/some/missing/file"].object_id, "Could not retrieve alias to self")
+    end
 
+    def test_aliases_are_added_to_class_and_catalog
+        resource = Puppet.type(:file).create(
+            :name => "/path/to/some/missing/file",
+            :ensure => "file"
+        )
+        resource.stubs(:path).returns("")
+
+        catalog = stub 'catalog'
+        catalog.stubs(:resource).returns(nil)
+        catalog.expects(:alias).with(resource, "funtest")
+        resource.catalog = catalog
+
+        assert_nothing_raised("Could not add alias") {
+            resource[:alias] = "funtest"
+        }
+
+        assert_equal(resource.object_id, Puppet.type(:file)["funtest"].object_id, "Could not retrieve alias")
+    end
+
+    def test_aliasing_fails_without_a_catalog
+        resource = Puppet.type(:file).create(
+            :name => "/no/such/file",
+            :ensure => "file"
+        )
+
+        assert_raise(Puppet::Error, "Did not fail to alias when no catalog was available") {
+            resource[:alias] = "funtest"
+        }
+    end
+
+    def test_catalogs_are_set_during_initialization_if_present_on_the_transobject
+        trans = Puppet::TransObject.new("/path/to/some/file", :file)
+        trans.catalog = :my_config
+        resource = trans.to_type
+        assert_equal(resource.catalog, trans.catalog, "Did not set catalog on initialization")
     end
 
     # Verify that requirements don't depend on file order
@@ -174,7 +207,7 @@ class TestType < Test::Unit::TestCase
             )
         }
 
-        comp = newcomp(twoobj, oneobj)
+        comp = mk_catalog(twoobj, oneobj)
 
         assert_nothing_raised {
             comp.finalize
@@ -246,124 +279,6 @@ class TestType < Test::Unit::TestCase
         # and make sure managed objects start with them
         assert(user.property(:ensure), "User did not get an ensure property")
     end
-
-    # Make sure removal works
-    def test_remove
-        objects = {}
-        top = Puppet.type(:component).create(:name => "top")
-        objects[top.class] = top
-
-        base = tempfile()
-
-        # now make a two-tier, 5 piece tree
-        %w{a b}.each do |letter|
-            name = "comp%s" % letter
-            comp = Puppet.type(:component).create(:name => name)
-            top.push comp
-            objects[comp.class] = comp
-
-            5.times do |i|
-                file = base + letter + i.to_s
-
-                obj = Puppet.type(:file).create(:name => file, :ensure => "file")
-
-                comp.push obj
-                objects[obj.class] = obj
-            end
-        end
-
-        assert_nothing_raised do
-            top.remove
-        end
-
-        objects.each do |klass, obj|
-            assert_nil(klass[obj.name], "object %s was not removed" % obj.name)
-        end
-    end
-
-    # Verify that objects can't be their own children.
-    def test_object_recursion
-        comp = Puppet.type(:component).create(:name => "top")
-
-        file = Puppet.type(:component).create(:name => "middle")
-
-        assert_raise(Puppet::DevError) do
-            comp.push(comp)
-        end
-
-        assert_raise(Puppet::DevError) do
-            file.push(file)
-        end
-
-        assert_raise(Puppet::DevError) do
-            comp.parent = comp
-        end
-
-        assert_raise(Puppet::DevError) do
-            file.parent = file
-        end
-
-        assert_nothing_raised {
-            comp.push(file)
-        }
-
-        assert_raise(Puppet::DevError) do
-            file.push(comp)
-        end
-
-        assert_raise(Puppet::DevError) do
-            comp.parent = file
-        end
-    end
-
-    def test_loadplugins
-        names = %w{loadedplugin1 loadplugin2 loadplugin3}
-        dirs = []
-        3.times { dirs << tempfile() }
-        # Set plugindest to something random
-        Puppet[:plugindest] = tempfile()
-
-        Puppet[:pluginpath] = dirs.join(":")
-
-        names.each do |name|
-            dir = dirs.shift
-            Dir.mkdir(dir)
-
-            # Create an extra file for later
-            [name, name + "2ness"].each do |n|
-                file = File.join(dir, n + ".rb")
-                File.open(file, "w") do |f|
-                    f.puts %{Puppet::Type.newtype('#{n}') do
-    newparam(:argument) do
-        isnamevar
-    end
-end
-}
-                end
-            end
-
-            assert(Puppet::Type.type(name),
-                "Did not get loaded plugin")
-
-            assert_nothing_raised {
-                Puppet::Type.type(name).create(
-                    :name => "myname"
-                )
-            }
-        end
-
-        # Now make sure the plugindest got added to our pluginpath
-        assert(Puppet[:pluginpath].split(":").include?(Puppet[:plugindest]),
-            "Plugin dest did not get added to plugin path")
-
-        # Now make sure it works with just a single path, using the extra files
-        # created above.
-        Puppet[:pluginpath] = Puppet[:pluginpath].split(":")[0]
-        assert(Puppet::Type.type("loadedplugin12ness"),
-            "Did not get loaded plugin")
-
-    end
-
     def test_newtype_methods
         assert_nothing_raised {
             Puppet::Type.newtype(:mytype) do
@@ -659,8 +574,8 @@ end
         assert_equal(greater, type.defaultprovider)
     end
 
-    # Make sure that we can have multiple isomorphic objects with the same name,
-    # but not with non-isomorphic objects.
+    # Make sure that we can have multiple non-isomorphic objects with the same name,
+    # but not with isomorphic objects.
     def test_isomorphic_names
         # First do execs, since they're not isomorphic.
         echo = Puppet::Util.binary "echo"
@@ -763,29 +678,23 @@ end
         assert_equal("Exec[yay]", exec.ref)
     end
     
-    def test_noop_metaparam
-        file = Puppet::Type.newfile :path => tempfile
-        assert(!file.noop, "file incorrectly in noop")
-        
-        assert_nothing_raised do
-            file[:noop] = true
-        end
-        assert(file.noop, "file should be in noop")
-        
-        # Now set the main one
-        Puppet[:noop] = true
-        assert(file.noop, "file should be in noop")
-        file[:noop] = false
-        assert(file.noop, "file should be in noop")
-    end
-    
     def test_path
+        config = mk_catalog
+
         # Check that our paths are built correctly.  Just pick a random, "normal" type.
         type = Puppet::Type.type(:exec)
         mk = Proc.new do |i, hash|
             hash[:title] = "exec%s" % i
             hash[:command] = "/bin/echo"
-            type.create(hash)
+            if parent = hash[:parent]
+                hash.delete(:parent)
+            end
+            res = type.create(hash)
+            config.add_resource res
+            if parent
+                config.add_edge(parent, res)
+            end
+            res
         end
         
         exec = mk.call(1, {})
@@ -793,25 +702,31 @@ end
         assert_equal("/Exec[exec1]", exec.path)
         
         comp = Puppet::Type.newcomponent :title => "My[component]", :type => "Yay"
+        config.add_resource comp
         
         exec = mk.call(2, :parent => comp)
         
         assert_equal("/My[component]/Exec[exec2]", exec.path)
         
         comp = Puppet::Type.newcomponent :name => "Other[thing]"
+        config.add_resource comp
         exec = mk.call(3, :parent => comp)
         assert_equal("/Other[thing]/Exec[exec3]", exec.path)
         
         comp = Puppet::Type.newcomponent :type => "server", :name => "server"
+        config.add_resource comp
         exec = mk.call(4, :parent => comp)
         assert_equal("/server/Exec[exec4]", exec.path)
         
-        comp = Puppet::Type.newcomponent :type => "whatever", :name => "main[top]"
+        comp = Puppet::Type.newcomponent :type => "whatever", :name => "class[main]"
+        config.add_resource comp
         exec = mk.call(5, :parent => comp)
         assert_equal("//Exec[exec5]", exec.path)
         
-        comp = Puppet::Type.newcomponent :type => "yay", :name => "Good[bad]", :parent => comp
-        exec = mk.call(6, :parent => comp)
+        newcomp = Puppet::Type.newcomponent :type => "yay", :name => "Good[bad]"
+        config.add_resource newcomp
+        config.add_edge comp, newcomp
+        exec = mk.call(6, :parent => newcomp)
         assert_equal("//Good[bad]/Exec[exec6]", exec.path)
     end
 
@@ -821,20 +736,62 @@ end
         end
         cleanup { Puppet::Type.rmtype(:faketype) }
         faketype.provide(:fake) do
-            def prefetch
-            end
         end
 
         obj = faketype.create :name => "yayness", :provider => :fake
         assert(obj, "did not create object")
 
-        obj.provider.expects(:prefetch)
         obj.expects(:retrieve)
         obj.expects(:propertychanges).returns([])
         obj.expects(:cache)
 
         obj.evaluate
     end
-end
 
-# $Id: type.rb 2500 2007-05-09 22:05:32Z luke $
+    # Partially test #704, but also cover the rest of the schedule management bases.
+    def test_schedule
+        Puppet::Type.type(:schedule).create(:name => "maint")
+
+        {"maint" => true, nil => false, :fail => :fail}.each do |name, should|
+            args = {:name => tempfile, :ensure => :file}
+            if name
+                args[:schedule] = name
+            end
+            resource = Puppet::Type.type(:file).create(args)
+
+            if should == :fail
+                assert_raise(Puppet::Error, "Did not fail on missing schedule") do
+                    resource.schedule
+                end
+            else
+                sched = nil
+                assert_nothing_raised("Failed when schedule was %s" % sched) do
+                    sched = resource.schedule
+                end
+
+                if should
+                    assert_equal(name, sched.name, "did not get correct schedule back")
+                end
+            end
+        end
+    end
+
+    # #801 -- resources only checked in noop should be rescheduled immediately.
+    def test_reschedule_when_noop
+        Puppet::Type.type(:schedule).mkdefaultschedules
+        file = Puppet::Type.type(:file).create(:path => "/tmp/whatever", :mode => "755", :noop => true, :schedule => :daily, :ensure => :file)
+
+        assert(file.noop?, "File not considered in noop")
+        assert(file.scheduled?, "File is not considered scheduled")
+
+        file.evaluate
+
+        assert_nil(file.cached(:checked), "Stored a checked time when running in noop mode when there were changes")
+        file.cache(:checked, nil)
+
+        file.stubs(:propertychanges).returns([])
+
+        file.evaluate
+        assert_instance_of(Time, file.cached(:checked), "Did not store a checked time when running in noop mode when there were no changes")
+    end
+end
