@@ -1,6 +1,6 @@
 #!/usr/bin/env ruby
 
-$:.unshift("../../lib") if __FILE__ =~ /\.rb$/
+require File.dirname(__FILE__) + '/../../lib/puppettest'
 
 require 'puppettest'
 require 'puppet/provider/nameservice'
@@ -11,7 +11,7 @@ class TestNameServiceProvider < Test::Unit::TestCase
 
     def test_option
         klass = Class.new(Puppet::Provider::NameService)
-        klass.model = Puppet::Type.type(:user)
+        klass.resource_type = Puppet::Type.type(:user)
 
         val = nil
         assert_nothing_raised {
@@ -30,4 +30,3 @@ class TestNameServiceProvider < Test::Unit::TestCase
     end
 end
 
-# $Id: nameservice.rb 2170 2007-02-07 17:21:52Z luke $
